#!/usr/bin/perl

use Cyrus::IMAP::Admin;
use Net::LDAP;
use strict;

# Global configuration variables

my $cyrus_user = 'admin';
my $cyrus_password = 'xxxxxxxxx';
my $cyrus_host = 'localhost';
my $ldap_user = 'cn=mta,dc=ufamail,dc=ru';
my $ldap_password = 'xxxxxxxxx';
my $ldap_host = 'ldapi://%2fvar%2frun%2fopenldap%2fldapi';

my @excluded_mailboxes = qw(admin@ufanet.ru admin@ufamail.ru);
my $default_domain = 'ufanet.ru';

# Cyrus connection

my $cyrus_conn = Cyrus::IMAP::Admin->new($cyrus_host) or die "$@";
$cyrus_conn->authenticate(User => $cyrus_user, Password => $cyrus_password);
die $cyrus_conn->error if $cyrus_conn->error;
my %cyrus_mailboxes = get_cyrus_mailboxes();

# LDAP connection

my $ldap_conn = Net::LDAP->new($ldap_host) or die "$@";
my $rc = $ldap_conn->bind($ldap_user, password => $ldap_password);
die $rc->error if $rc->code;
my %ldap_mailboxes = get_ldap_mailboxes();
$rc = $ldap_conn->unbind;
die $rc->error if $rc->code;
undef $ldap_conn;
undef $rc;

# Delete excluded mailboxes from both lists

foreach (@excluded_mailboxes) {
	$_ = 'user.' . $_;
	delete($cyrus_mailboxes{$_}) if exists $cyrus_mailboxes{$_};
	delete($ldap_mailboxes{$_}) if exists $ldap_mailboxes{$_};
}

# Delete mailboxes not existing in LDAP

foreach (keys %cyrus_mailboxes) {
	unless (exists $ldap_mailboxes{$_}) {
		$cyrus_conn->setacl($_, $cyrus_user => 'c');
		die $cyrus_conn->error if $cyrus_conn->error;
		$cyrus_conn->delete($_);
		die $cyrus_conn->error if $cyrus_conn->error;
	}
}

# Add new mailboxes and setup quotas

foreach (keys %ldap_mailboxes) {
	unless (exists $cyrus_mailboxes{$_}) {
		$cyrus_conn->create($_);
		die $cyrus_conn->error if $cyrus_conn->error;
	}
	if ($ldap_mailboxes{$_}) {
		$cyrus_conn->setquota($_, 'STORAGE', $ldap_mailboxes{$_});
	} else {
		$cyrus_conn->setquota($_);
	}
	die $cyrus_conn->error if $cyrus_conn->error;
}

exit;

# Subroutines

sub get_cyrus_mailboxes {
	my %rc = ();
	my $mbox = '';
	my $root = '';
	my %quota = ();
	my @mailboxes = $cyrus_conn->list('*');

	die $cyrus_conn->error if $cyrus_conn->error;

	foreach (@mailboxes) {
		$mbox = $$_[0];
		$mbox .= "\@$default_domain" unless $mbox =~ /\@/;
		next if $mbox =~ /\w+\.\w+\.[^@]+\@.*/;
		next if $mbox !~ /^user\..*/;
		($root, %quota) = $cyrus_conn->listquotaroot($mbox);
		die $cyrus_conn->error if $cyrus_conn->error;
		$rc{$mbox} = $quota{'STORAGE'}[1];
	}
	return %rc;
}

sub get_ldap_mailboxes {
	my %rc = ();
	my $mesg = $ldap_conn->search(
		base => 'dc=ufamail,dc=ru',
		filter => '(objectClass=mailAccount)',
		attrs => ['mailBox', 'boxQuota']
		);

	die $mesg->error if $mesg->code;

	foreach ($mesg->entries) {
		$rc{'user.' . lc($_->get_value('mailBox'))} = $_->get_value('boxQuota') || 0;
	}
	return %rc;
}
